#include "command_registry.hpp"

CommandRegistry& CommandRegistry::instance() {
    static CommandRegistry reg;
    return reg;
}

void CommandRegistry::register_command(const CommandInfo& info) {
    auto ptr = std::make_shared<CommandInfo>(info);
    for (const auto& name : info.names) {
        command_map_[name] = ptr;
    }
    all_commands_.push_back(ptr);
}

const CommandInfo* CommandRegistry::find_command(const std::string& name) const {
    auto it = command_map_.find(name);
    if (it != command_map_.end()) return it->second.get();
    return nullptr;
}

std::vector<std::string> CommandRegistry::list_commands(bool include_hidden) const {
    std::set<std::string> out;
    for (const auto& cmd : all_commands_) {
        if (!cmd->hidden || include_hidden) {
            for (const auto& name : cmd->names) out.insert(name);
        }
    }
    return std::vector<std::string>(out.begin(), out.end());
}

std::vector<std::string> CommandRegistry::list_primary_commands(bool include_hidden) const {
    std::set<std::string> out;
    for (const auto& cmd : all_commands_) {
        if (!cmd->hidden || include_hidden) {
            if (cmd->names.size() > 0)
            out.insert(cmd->names[0]);
        }
    }
    return std::vector<std::string>(out.begin(), out.end());
}


void CommandRegistry::autocomplete(const std::vector<std::string>& args) const {
    // dropshell autocomplete <command> <arg> <arg> ...
    if (args.size() < 3) {
        for (const auto& name : list_primary_commands(false)) {
            std::cout << name << std::endl;
        }
        return;
    }
    std::string cmd = args[2];
    auto* info = find_command(cmd);
    if (info && info->autocomplete) {
        CommandContext ctx{args[0], cmd, std::vector<std::string>(args.begin() + 2, args.end())};
        info->autocomplete(ctx);
    }
}